;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'i18n '("mcharset" "mcs-20" "mcs-xm" "mcs-xmu"))

;;; custom-load.el ends here
